@@ -1,8 +1,8 @@
 # Natural Language Toolkit: Regexp Chunk Parser Application
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Edward Loper <edloper@gradient.cis.upenn.edu>
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2015 NLTK Project
+# Author: Edward Loper <edloper@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -15,13 +15,14 @@ parser ``nltk.chunk.RegexpChunkParser``.
 # configuration parameters to select what's being chunked (eg VP vs NP)
 # and what part of the data is being used as the development set.
 
+import nltk.compat
 import time
 import textwrap
 import re
 import random
-import tkFileDialog, tkFont
+import tkinter.filedialog, tkinter.font
 
-from Tkinter import (Button, Canvas, Checkbutton,
+from tkinter import (Button, Canvas, Checkbutton,
                      Frame, IntVar, Label, Menu,
                      Scrollbar, Text, Tk)
 
@@ -256,7 +257,7 @@ class RegexpChunkApp(object):
         return grammar
 
     def __init__(self, devset_name='conll2000', devset=None,
-                 grammar = '', chunk_node='NP', tagset=None):
+                 grammar = '', chunk_label='NP', tagset=None):
         """
         :param devset_name: The name of the development set; used for
             display & for save files.  If either the name 'treebank'
@@ -267,7 +268,7 @@ class RegexpChunkApp(object):
         :param tagset: Dictionary from tags to string descriptions, used
             for the help page.  Defaults to ``self.TAGSET``.
         """
-        self._chunk_node = chunk_node
+        self._chunk_label = chunk_label
 
         if tagset is None: tagset = self.TAGSET
         self.tagset = tagset
@@ -325,7 +326,7 @@ class RegexpChunkApp(object):
         """The index of the next sentence in the development set that
            should be looked at by the eval demon."""
 
-        self._eval_score = ChunkScore(chunk_node=chunk_node)
+        self._eval_score = ChunkScore(chunk_label=chunk_label)
         """The ``ChunkScore`` object that's used to keep track of the score
         of the current grammar on the development set."""
 
@@ -374,10 +375,10 @@ class RegexpChunkApp(object):
         # TWhat's our font size (default=same as sysfont)
         self._size = IntVar(top)
         self._size.set(20)
-        self._font = tkFont.Font(family='helvetica',
+        self._font = tkinter.font.Font(family='helvetica',
                                  size=-self._size.get())
-        self._smallfont = tkFont.Font(family='helvetica',
-                                      size=-(self._size.get()*14/20))
+        self._smallfont = tkinter.font.Font(family='helvetica',
+                                      size=-(int(self._size.get()*14/20)))
 
     def _init_menubar(self, parent):
         menubar = Menu(parent)
@@ -577,7 +578,7 @@ class RegexpChunkApp(object):
                     self._eval_normalized_grammar = None
                     return
             self._eval_index = 0
-            self._eval_score = ChunkScore(chunk_node=self._chunk_node)
+            self._eval_score = ChunkScore(chunk_label=self._chunk_label)
             self._eval_grammar = self.grammar
             self._eval_normalized_grammar = self.normalized_grammar
 
@@ -854,8 +855,8 @@ class RegexpChunkApp(object):
         for (name, tabstops, text) in self.HELP:
             if name == tab:
                 text = text.replace('<<TAGSET>>', '\n'.join(
-                    ('\t%s\t%s' % item for item in sorted(self.tagset.items(),
-                    key=lambda (t,w):re.match('\w+',t) and (0,t) or (1,t)))))
+                    ('\t%s\t%s' % item for item in sorted(list(self.tagset.items()),
+                    key=lambda t_w:re.match('\w+',t_w[0]) and (0,t_w[0]) or (1,t_w[0])))))
 
                 self.helptabs[name].config(**self._HELPTAB_FG_PARAMS)
                 self.helpbox.config(tabs=tabstops)
@@ -901,7 +902,7 @@ class RegexpChunkApp(object):
         self.normalized_grammar = self.normalize_grammar(
             self._history[index][0])
         if self.normalized_grammar:
-            rules = [RegexpChunkRule.parse(line)
+            rules = [RegexpChunkRule.fromstring(line)
                      for line in self.normalized_grammar.split('\n')]
         else:
             rules = []
@@ -992,7 +993,7 @@ class RegexpChunkApp(object):
         wordnum = 0
         for child in tree:
             if isinstance(child, Tree):
-                if child.node == self._chunk_node:
+                if child.label() == self._chunk_label:
                     chunks.add( (wordnum, wordnum+len(child)) )
                 wordnum += len(child)
             else:
@@ -1034,8 +1035,8 @@ class RegexpChunkApp(object):
             line = line.strip()
             if line:
                 try:
-                    RegexpChunkRule.parse(line)
-                except ValueError, e:
+                    RegexpChunkRule.fromstring(line)
+                except ValueError as e:
                     self.grammarbox.tag_add('error', '%s.0' % (lineno+1),
                                             '%s.0 lineend' % (lineno+1))
         self.status['text'] = ''
@@ -1067,11 +1068,11 @@ class RegexpChunkApp(object):
         try:
             # Note: the normalized grammar has no blank lines.
             if normalized_grammar:
-                rules = [RegexpChunkRule.parse(line)
+                rules = [RegexpChunkRule.fromstring(line)
                          for line in normalized_grammar.split('\n')]
             else:
                 rules = []
-        except ValueError, e:
+        except ValueError as e:
             # Use the un-normalized grammar for error highlighting.
             self._grammarcheck(grammar)
             self.chunker = None
@@ -1115,7 +1116,7 @@ class RegexpChunkApp(object):
     def _chunkparse(self, words):
         try:
             return self.chunker.parse(words)
-        except (ValueError, IndexError), e:
+        except (ValueError, IndexError) as e:
             # There's an error somewhere in the grammar, but we're not sure
             # exactly where, so just mark the whole grammar as bad.
             # E.g., this is caused by: "({<NN>})"
@@ -1157,7 +1158,7 @@ class RegexpChunkApp(object):
         if not filename:
             ftypes = [('Chunk Gramamr', '.chunk'),
                       ('All files', '*')]
-            filename = tkFileDialog.asksaveasfilename(filetypes=ftypes,
+            filename = tkinter.filedialog.asksaveasfilename(filetypes=ftypes,
                                                       defaultextension='.chunk')
             if not filename: return
         if (self._history and self.normalized_grammar ==
@@ -1169,23 +1170,23 @@ class RegexpChunkApp(object):
         else:
             precision = recall = fscore = 'Not finished evaluation yet'
 
-        out = open(filename, 'w')
-        out.write(self.SAVE_GRAMMAR_TEMPLATE % dict(
-            date=time.ctime(), devset=self.devset_name,
-            precision=precision, recall=recall, fscore=fscore,
-            grammar=self.grammar.strip()))
-        out.close()
+        with open(filename, 'w') as outfile:
+            outfile.write(self.SAVE_GRAMMAR_TEMPLATE % dict(
+                date=time.ctime(), devset=self.devset_name,
+                precision=precision, recall=recall, fscore=fscore,
+                grammar=self.grammar.strip()))
 
     def load_grammar(self, filename=None):
         if not filename:
             ftypes = [('Chunk Gramamr', '.chunk'),
                       ('All files', '*')]
-            filename = tkFileDialog.askopenfilename(filetypes=ftypes,
+            filename = tkinter.filedialog.askopenfilename(filetypes=ftypes,
                                                     defaultextension='.chunk')
             if not filename: return
         self.grammarbox.delete('1.0', 'end')
         self.update()
-        grammar = open(filename).read()
+        with open(filename, 'r') as infile:
+            grammar = infile.read()
         grammar = re.sub('^\# Regexp Chunk Parsing Grammar[\s\S]*'
                          'F-score:.*\n', '', grammar).lstrip()
         self.grammarbox.insert('1.0', grammar)
@@ -1195,37 +1196,36 @@ class RegexpChunkApp(object):
         if not filename:
             ftypes = [('Chunk Gramamr History', '.txt'),
                       ('All files', '*')]
-            filename = tkFileDialog.asksaveasfilename(filetypes=ftypes,
+            filename = tkinter.filedialog.asksaveasfilename(filetypes=ftypes,
                                                       defaultextension='.txt')
             if not filename: return
 
-        out = open(filename, 'w')
-        out.write('# Regexp Chunk Parsing Grammar History\n')
-        out.write('# Saved %s\n' % time.ctime())
-        out.write('# Development set: %s\n' % self.devset_name)
-        for i, (g, p, r, f) in enumerate(self._history):
-            hdr = ('Grammar %d/%d (precision=%.2f%%, recall=%.2f%%, '
-                   'fscore=%.2f%%)' % (i+1, len(self._history),
-                                       p*100, r*100, f*100))
-            out.write('\n%s\n' % hdr)
-            out.write(''.join('  %s\n' % line for line in g.strip().split()))
+        with open(filename, 'w') as outfile:
+            outfile.write('# Regexp Chunk Parsing Grammar History\n')
+            outfile.write('# Saved %s\n' % time.ctime())
+            outfile.write('# Development set: %s\n' % self.devset_name)
+            for i, (g, p, r, f) in enumerate(self._history):
+                hdr = ('Grammar %d/%d (precision=%.2f%%, recall=%.2f%%, '
+                       'fscore=%.2f%%)' % (i+1, len(self._history),
+                                           p*100, r*100, f*100))
+                outfile.write('\n%s\n' % hdr)
+                outfile.write(''.join('  %s\n' % line for line in g.strip().split()))
 
-        if not (self._history and self.normalized_grammar ==
-                self.normalize_grammar(self._history[-1][0])):
-            if self.chunker is None:
-                out.write('\nCurrent Grammar (not well-formed)\n')
-            else:
-                out.write('\nCurrent Grammar (not evaluated)\n')
-            out.write(''.join('  %s\n' % line for line
-                              in self.grammar.strip().split()))
-        out.close()
+            if not (self._history and self.normalized_grammar ==
+                    self.normalize_grammar(self._history[-1][0])):
+                if self.chunker is None:
+                    outfile.write('\nCurrent Grammar (not well-formed)\n')
+                else:
+                    outfile.write('\nCurrent Grammar (not evaluated)\n')
+                outfile.write(''.join('  %s\n' % line for line
+                                  in self.grammar.strip().split()))
 
     def about(self, *e):
         ABOUT = ("NLTK RegExp Chunk Parser Application\n"+
                  "Written by Edward Loper")
         TITLE = 'About: Regular Expression Chunk Parser Application'
         try:
-            from tkMessageBox import Message
+            from tkinter.messagebox import Message
             Message(message=ABOUT, title=TITLE).show()
         except:
             ShowText(self.top, TITLE, ABOUT)
