@@ -1,24 +1,26 @@
 # Natural Language Toolkit: API for Corpus Readers
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Steven Bird <sb@ldc.upenn.edu>
-#         Edward Loper <edloper@gradient.cis.upenn.edu>
-# URL: <http://www.nltk.org/>
+# Copyright (C) 2001-2015 NLTK Project
+# Author: Steven Bird <stevenbird1@gmail.com>
+#         Edward Loper <edloper@gmail.com>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
 API for corpus readers.
 """
+from __future__ import unicode_literals
 
 import os
 import re
 from collections import defaultdict
 
+from nltk import compat
 from nltk.data import PathPointer, FileSystemPathPointer, ZipFilePathPointer
-from nltk.sourcedstring import SourcedStringStream
 
-from util import *
+from nltk.corpus.reader.util import *
 
+@compat.python_2_unicode_compatible
 class CorpusReader(object):
     """
     A base class for "corpus reader" classes, each of which can be
@@ -37,7 +39,8 @@ class CorpusReader(object):
     selection arguments, such as ``fileids`` or ``categories``, which can
     be used to select which portion of the corpus should be returned.
     """
-    def __init__(self, root, fileids, encoding=None, tag_mapping_function=None):
+
+    def __init__(self, root, fileids, encoding='utf8', tagset=None):
         """
         :type root: PathPointer or str
         :param root: A path pointer identifying the root directory for
@@ -64,12 +67,12 @@ class CorpusReader(object):
               using non-unicode byte strings.
             - None: the file contents of all files will be
               processed using non-unicode byte strings.
-        :param tag_mapping_function: A function for normalizing or
-                simplifying the POS tags returned by the tagged_words()
-                or tagged_sents() methods.
+        :param tagset: The name of the tagset used by this corpus, to be used
+              for normalizing or converting the POS tags returned by the
+              tagged_...() methods.
         """
         # Convert the root to a path pointer, if necessary.
-        if isinstance(root, basestring):
+        if isinstance(root, compat.string_types) and not isinstance(root, PathPointer):
             m = re.match('(.*\.zip)/?(.*)$|', root)
             zipfile, zipentry = m.groups()
             if zipfile:
@@ -80,7 +83,7 @@ class CorpusReader(object):
             raise TypeError('CorpusReader: expected a string or a PathPointer')
 
         # If `fileids` is a regexp, then expand it.
-        if isinstance(fileids, basestring):
+        if isinstance(fileids, compat.string_types):
             fileids = find_corpus_fileids(root, fileids)
 
         self._fileids = fileids
@@ -105,8 +108,8 @@ class CorpusReader(object):
         self._encoding = encoding
         """The default unicode encoding for the fileids that make up
            this corpus.  If ``encoding`` is None, then the file
-           contents are processed using byte strings (str)."""
-        self._tag_mapping_function = tag_mapping_function
+           contents are processed using byte strings."""
+        self._tagset = tagset
 
     def __repr__(self):
         if isinstance(self._root, ZipFilePathPointer):
@@ -115,11 +118,19 @@ class CorpusReader(object):
             path = '%s' % self._root.path
         return '<%s in %r>' % (self.__class__.__name__, path)
 
+    def ensure_loaded(self):
+        """
+        Load this corpus (if it has not already been loaded).  This is
+        used by LazyCorpusLoader as a simple method that can be used to
+        make sure a corpus is loaded -- e.g., in case a user wants to
+        do help(some_corpus).
+        """
+        pass # no need to actually do anything.
+
     def readme(self):
         """
         Return the contents of the corpus README file, if it exists.
         """
-
         return self.open("README").read()
 
     def fileids(self):
@@ -161,21 +172,21 @@ class CorpusReader(object):
         """
         if fileids is None:
             fileids = self._fileids
-        elif isinstance(fileids, basestring):
+        elif isinstance(fileids, compat.string_types):
             fileids = [fileids]
 
         paths = [self._root.join(f) for f in fileids]
 
         if include_encoding and include_fileid:
-            return zip(paths, [self.encoding(f) for f in fileids], fileids)
+            return list(zip(paths, [self.encoding(f) for f in fileids], fileids))
         elif include_fileid:
-            return zip(paths, fileids)
+            return list(zip(paths, fileids))
         elif include_encoding:
-            return zip(paths, [self.encoding(f) for f in fileids])
+            return list(zip(paths, [self.encoding(f) for f in fileids]))
         else:
             return paths
 
-    def open(self, file, sourced=False):
+    def open(self, file):
         """
         Return an open stream that can be used to read the given file.
         If the file's encoding is not None, then the stream will
@@ -185,8 +196,6 @@ class CorpusReader(object):
         """
         encoding = self.encoding(file)
         stream = self._root.join(file).open(encoding)
-        if sourced:
-            stream = SourcedStringStream(stream, file)
         return stream
 
     def encoding(self, file):
@@ -317,7 +326,7 @@ class CategorizedCorpusReader(object):
             self._init()
         if fileids is None:
             return sorted(self._c2f)
-        if isinstance(fileids, basestring):
+        if isinstance(fileids, compat.string_types):
             fileids = [fileids]
         return sorted(set.union(*[self._f2c[d] for d in fileids]))
 
@@ -328,7 +337,7 @@ class CategorizedCorpusReader(object):
         """
         if categories is None:
             return super(CategorizedCorpusReader, self).fileids()
-        elif isinstance(categories, basestring):
+        elif isinstance(categories, compat.string_types):
             if self._f2c is None:
                 self._init()
             if categories in self._c2f:
@@ -370,7 +379,7 @@ class SyntaxCorpusReader(CorpusReader):
 
     def raw(self, fileids=None):
         if fileids is None: fileids = self._fileids
-        elif isinstance(fileids, basestring): fileids = [fileids]
+        elif isinstance(fileids, compat.string_types): fileids = [fileids]
         return concat([self.open(f).read() for f in fileids])
 
     def parsed_sents(self, fileids=None):
@@ -378,9 +387,9 @@ class SyntaxCorpusReader(CorpusReader):
         return concat([StreamBackedCorpusView(fileid, reader, encoding=enc)
                        for fileid, enc in self.abspaths(fileids, True)])
 
-    def tagged_sents(self, fileids=None, simplify_tags=False):
+    def tagged_sents(self, fileids=None, tagset=None):
         def reader(stream):
-            return self._read_tagged_sent_block(stream, simplify_tags)
+            return self._read_tagged_sent_block(stream, tagset)
         return concat([StreamBackedCorpusView(fileid, reader, encoding=enc)
                        for fileid, enc in self.abspaths(fileids, True)])
 
@@ -389,9 +398,9 @@ class SyntaxCorpusReader(CorpusReader):
         return concat([StreamBackedCorpusView(fileid, reader, encoding=enc)
                        for fileid, enc in self.abspaths(fileids, True)])
 
-    def tagged_words(self, fileids=None, simplify_tags=False):
+    def tagged_words(self, fileids=None, tagset=None):
         def reader(stream):
-            return self._read_tagged_word_block(stream, simplify_tags)
+            return self._read_tagged_word_block(stream, tagset)
         return concat([StreamBackedCorpusView(fileid, reader, encoding=enc)
                        for fileid, enc in self.abspaths(fileids, True)])
 
@@ -407,18 +416,18 @@ class SyntaxCorpusReader(CorpusReader):
     def _read_word_block(self, stream):
         return sum(self._read_sent_block(stream), [])
 
-    def _read_tagged_word_block(self, stream, simplify_tags=False):
-        return sum(self._read_tagged_sent_block(stream, simplify_tags), [])
+    def _read_tagged_word_block(self, stream, tagset=None):
+        return sum(self._read_tagged_sent_block(stream, tagset), [])
 
     def _read_sent_block(self, stream):
-        return filter(None, [self._word(t) for t in self._read_block(stream)])
+        return list(filter(None, [self._word(t) for t in self._read_block(stream)]))
 
-    def _read_tagged_sent_block(self, stream, simplify_tags=False):
-        return filter(None, [self._tag(t, simplify_tags)
-                             for t in self._read_block(stream)])
+    def _read_tagged_sent_block(self, stream, tagset=None):
+        return list(filter(None, [self._tag(t, tagset)
+                             for t in self._read_block(stream)]))
 
     def _read_parsed_sent_block(self, stream):
-        return filter(None, [self._parse(t) for t in self._read_block(stream)])
+        return list(filter(None, [self._parse(t) for t in self._read_block(stream)]))
 
     #} End of Block Readers
     #------------------------------------------------------------
