@@ -1,10 +1,10 @@
 # Natural Language Toolkit: Sequential Backoff Taggers
 #
-# Copyright (C) 2001-2012 NLTK Project
-# Author: Edward Loper <edloper@gradient.cis.upenn.edu>
-#         Steven Bird <sb@csse.unimelb.edu.au> (minor additions)
+# Copyright (C) 2001-2015 NLTK Project
+# Author: Edward Loper <edloper@gmail.com>
+#         Steven Bird <stevenbird1@gmail.com> (minor additions)
 #         Tiago Tresoldi <tresoldi@users.sf.net> (original affix tagger)
-# URL: <http://www.nltk.org/>
+# URL: <http://nltk.org/>
 # For license information, see LICENSE.TXT
 
 """
@@ -17,14 +17,18 @@ determine a tag for the specified token, then its backoff tagger is
 consulted instead.  Any SequentialBackoffTagger may serve as a
 backoff tagger for any other SequentialBackoffTagger.
 """
+from __future__ import print_function, unicode_literals
 
-import re, yaml
+import re
 
-from nltk.probability import FreqDist, ConditionalFreqDist
-from nltk.classify.naivebayes import NaiveBayesClassifier
+from nltk.probability import ConditionalFreqDist
+from nltk.classify import NaiveBayesClassifier
+from nltk.compat import python_2_unicode_compatible
 
 from nltk.tag.api import TaggerI, FeaturesetTaggerI
 
+from nltk import jsontags
+
 ######################################################################
 #{ Abstract Base Classes
 ######################################################################
@@ -48,15 +52,14 @@ class SequentialBackoffTagger(TaggerI):
     @property
     def backoff(self):
         """The backoff tagger for this tagger."""
-        if len(self._taggers) < 2: return None
-        else: return self._taggers[1]
+        return self._taggers[1] if len(self._taggers) > 1 else None
 
     def tag(self, tokens):
         # docs inherited from TaggerI
         tags = []
         for i in range(len(tokens)):
             tags.append(self.tag_one(tokens, i, tags))
-        return zip(tokens, tags)
+        return list(zip(tokens, tags))
 
     def tag_one(self, tokens, index, history):
         """
@@ -99,6 +102,7 @@ class SequentialBackoffTagger(TaggerI):
         raise NotImplementedError()
 
 
+@python_2_unicode_compatible
 class ContextTagger(SequentialBackoffTagger):
     """
     An abstract base class for sequential backoff taggers that choose
@@ -119,10 +123,7 @@ class ContextTagger(SequentialBackoffTagger):
         :param backoff: The backoff tagger that should be used for this tagger.
         """
         SequentialBackoffTagger.__init__(self, backoff)
-        if context_to_tag:
-            self._context_to_tag = context_to_tag
-        else:
-            self._context_to_tag = {}
+        self._context_to_tag = (context_to_tag if context_to_tag else {})
 
     def context(self, tokens, index, history):
         """
@@ -180,7 +181,7 @@ class ContextTagger(SequentialBackoffTagger):
                 token_count += 1
                 context = self.context(tokens, index, tags[:index])
                 if context is None: continue
-                fd[context].inc(tag)
+                fd[context][tag] += 1
                 # If the backoff got it wrong, this context is useful:
                 if (self.backoff is None or
                     tag != self.backoff.tag_one(tokens, index, tags[:index])):
@@ -201,21 +202,23 @@ class ContextTagger(SequentialBackoffTagger):
             size = len(self._context_to_tag)
             backoff = 100 - (hit_count * 100.0)/ token_count
             pruning = 100 - (size * 100.0) / len(fd.conditions())
-            print "[Trained Unigram tagger:",
-            print "size=%d, backoff=%.2f%%, pruning=%.2f%%]" % (
-                size, backoff, pruning)
+            print("[Trained Unigram tagger:", end=' ')
+            print("size=%d, backoff=%.2f%%, pruning=%.2f%%]" % (
+                size, backoff, pruning))
 
 ######################################################################
 #{ Tagger Classes
 ######################################################################
 
-class DefaultTagger(SequentialBackoffTagger, yaml.YAMLObject):
+@python_2_unicode_compatible
+@jsontags.register_tag
+class DefaultTagger(SequentialBackoffTagger):
     """
     A tagger that assigns the same tag to every token.
 
-        >>> from nltk.tag.sequential import DefaultTagger
+        >>> from nltk.tag import DefaultTagger
         >>> default_tagger = DefaultTagger('NN')
-        >>> default_tagger.tag('This is a test'.split())
+        >>> list(default_tagger.tag('This is a test'.split()))
         [('This', 'NN'), ('is', 'NN'), ('a', 'NN'), ('test', 'NN')]
 
     This tagger is recommended as a backoff tagger, in cases where
@@ -225,12 +228,21 @@ class DefaultTagger(SequentialBackoffTagger, yaml.YAMLObject):
     :param tag: The tag to assign to each token
     :type tag: str
     """
-    yaml_tag = '!nltk.DefaultTagger'
+
+    json_tag = 'nltk.tag.sequential.DefaultTagger'
 
     def __init__(self, tag):
         self._tag = tag
         SequentialBackoffTagger.__init__(self, None)
 
+    def encode_json_obj(self):
+        return self._tag
+
+    @classmethod
+    def decode_json_obj(cls, obj):
+        tag = obj
+        return cls(tag)
+
     def choose_tag(self, tokens, index, history):
         return self._tag  # ignore token and history
 
@@ -238,7 +250,8 @@ class DefaultTagger(SequentialBackoffTagger, yaml.YAMLObject):
         return '<DefaultTagger: tag=%s>' % self._tag
 
 
-class NgramTagger(ContextTagger, yaml.YAMLObject):
+@jsontags.register_tag
+class NgramTagger(ContextTagger):
     """
     A tagger that chooses a token's tag based on its word string and
     on the preceding n word's tags.  In particular, a tuple
@@ -261,7 +274,7 @@ class NgramTagger(ContextTagger, yaml.YAMLObject):
         fewer than *cutoff* times, then exclude it from the
         context-to-tag table for the new tagger.
     """
-    yaml_tag = '!nltk.NgramTagger'
+    json_tag = 'nltk.tag.sequential.NgramTagger'
 
     def __init__(self, n, train=None, model=None,
                  backoff=None, cutoff=0, verbose=False):
@@ -273,11 +286,20 @@ class NgramTagger(ContextTagger, yaml.YAMLObject):
         if train:
             self._train(train, cutoff, verbose)
 
+    def encode_json_obj(self):
+        return self._n, self._context_to_tag, self.backoff
+
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _n, _context_to_tag, backoff = obj
+        return cls(_n, model=_context_to_tag, backoff=backoff)
+
     def context(self, tokens, index, history):
         tag_context = tuple(history[max(0,index-self._n+1):index])
-        return (tag_context, tokens[index])
+        return tag_context, tokens[index]
 
 
+@jsontags.register_tag
 class UnigramTagger(NgramTagger):
     """
     Unigram Tagger
@@ -286,16 +308,17 @@ class UnigramTagger(NgramTagger):
     corpus, and then uses that information to assign tags to new tokens.
 
         >>> from nltk.corpus import brown
-        >>> from nltk.tag.sequential import UnigramTagger
+        >>> from nltk.tag import UnigramTagger
         >>> test_sent = brown.sents(categories='news')[0]
         >>> unigram_tagger = UnigramTagger(brown.tagged_sents(categories='news')[:500])
-        >>> unigram_tagger.tag(test_sent)
-        [('The', 'AT'), ('Fulton', 'NP-TL'), ('County', 'NN-TL'), ('Grand', 'JJ-TL'),
-        ('Jury', 'NN-TL'), ('said', 'VBD'), ('Friday', 'NR'), ('an', 'AT'),
-        ('investigation', 'NN'), ('of', 'IN'), ("Atlanta's", 'NP$'), ('recent', 'JJ'),
-        ('primary', 'NN'), ('election', 'NN'), ('produced', 'VBD'), ('``', '``'),
-        ('no', 'AT'), ('evidence', 'NN'), ("''", "''"), ('that', 'CS'), ('any', 'DTI'),
-        ('irregularities', 'NNS'), ('took', 'VBD'), ('place', 'NN'), ('.', '.')]
+        >>> for tok, tag in unigram_tagger.tag(test_sent):
+        ...     print("(%s, %s), " % (tok, tag))
+        (The, AT), (Fulton, NP-TL), (County, NN-TL), (Grand, JJ-TL),
+        (Jury, NN-TL), (said, VBD), (Friday, NR), (an, AT),
+        (investigation, NN), (of, IN), (Atlanta's, NP$), (recent, JJ),
+        (primary, NN), (election, NN), (produced, VBD), (``, ``),
+        (no, AT), (evidence, NN), ('', ''), (that, CS), (any, DTI),
+        (irregularities, NNS), (took, VBD), (place, NN), (., .),
 
     :param train: The corpus of training data, a list of tagged sentences
     :type train: list(list(tuple(str, str)))
@@ -309,17 +332,26 @@ class UnigramTagger(NgramTagger):
     :type cutoff: int
     """
 
-    yaml_tag = '!nltk.UnigramTagger'
+    json_tag = 'nltk.tag.sequential.UnigramTagger'
 
     def __init__(self, train=None, model=None,
                  backoff=None, cutoff=0, verbose=False):
         NgramTagger.__init__(self, 1, train, model,
                              backoff, cutoff, verbose)
 
+    def encode_json_obj(self):
+        return self._context_to_tag, self.backoff
+
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _context_to_tag, backoff = obj
+        return cls(model=_context_to_tag, backoff=backoff)
+
     def context(self, tokens, index, history):
         return tokens[index]
 
 
+@jsontags.register_tag
 class BigramTagger(NgramTagger):
     """
     A tagger that chooses a token's tag based its word string and on
@@ -338,14 +370,23 @@ class BigramTagger(NgramTagger):
         in order not to use the backoff tagger
     :type cutoff: int
     """
-    yaml_tag = '!nltk.BigramTagger'
+    json_tag = 'nltk.tag.sequential.BigramTagger'
 
-    def __init__(self, train, model=None,
+    def __init__(self, train=None, model=None,
                  backoff=None, cutoff=0, verbose=False):
         NgramTagger.__init__(self, 2, train, model,
                              backoff, cutoff, verbose)
 
+    def encode_json_obj(self):
+        return self._context_to_tag, self.backoff
 
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _context_to_tag, backoff = obj
+        return cls(model=_context_to_tag, backoff=backoff)
+
+
+@jsontags.register_tag
 class TrigramTagger(NgramTagger):
     """
     A tagger that chooses a token's tag based its word string and on
@@ -364,15 +405,24 @@ class TrigramTagger(NgramTagger):
         in order not to use the backoff tagger
     :type cutoff: int
     """
-    yaml_tag = '!nltk.TrigramTagger'
+    json_tag = 'nltk.tag.sequential.TrigramTagger'
 
     def __init__(self, train=None, model=None,
                  backoff=None, cutoff=0, verbose=False):
         NgramTagger.__init__(self, 3, train, model,
                              backoff, cutoff, verbose)
 
+    def encode_json_obj(self):
+        return self._context_to_tag, self.backoff
 
-class AffixTagger(ContextTagger, yaml.YAMLObject):
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _context_to_tag, backoff = obj
+        return cls(model=_context_to_tag, backoff=backoff)
+
+
+@jsontags.register_tag
+class AffixTagger(ContextTagger):
     """
     A tagger that chooses a token's tag based on a leading or trailing
     substring of its word string.  (It is important to note that these
@@ -391,7 +441,7 @@ class AffixTagger(ContextTagger, yaml.YAMLObject):
         tag of None by this tagger.
     """
 
-    yaml_tag = '!nltk.AffixTagger'
+    json_tag = 'nltk.tag.sequential.AffixTagger'
 
     def __init__(self, train=None, model=None, affix_length=-3,
                  min_stem_length=2, backoff=None, cutoff=0, verbose=False):
@@ -406,6 +456,19 @@ class AffixTagger(ContextTagger, yaml.YAMLObject):
         if train:
             self._train(train, cutoff, verbose)
 
+    def encode_json_obj(self):
+        return self._affix_length, self._min_word_length, self._context_to_tag, self.backoff
+
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _affix_length, _min_word_length, _context_to_tag, backoff = obj
+        return cls(
+            affix_length=_affix_length,
+            min_stem_length=_min_word_length - abs(_affix_length),
+            model=_context_to_tag,
+            backoff=backoff
+        )
+
     def context(self, tokens, index, history):
         token = tokens[index]
         if len(token) < self._min_word_length:
@@ -416,7 +479,9 @@ class AffixTagger(ContextTagger, yaml.YAMLObject):
             return token[self._affix_length:]
 
 
-class RegexpTagger(SequentialBackoffTagger, yaml.YAMLObject):
+@python_2_unicode_compatible
+@jsontags.register_tag
+class RegexpTagger(SequentialBackoffTagger):
     """
     Regular Expression Tagger
 
@@ -426,7 +491,7 @@ class RegexpTagger(SequentialBackoffTagger, yaml.YAMLObject):
     of speech tag:
 
         >>> from nltk.corpus import brown
-        >>> from nltk.tag.sequential import RegexpTagger
+        >>> from nltk.tag import RegexpTagger
         >>> test_sent = brown.sents(categories='news')[0]
         >>> regexp_tagger = RegexpTagger(
         ...     [(r'^-?[0-9]+(.[0-9]+)?$', 'CD'),   # cardinal numbers
@@ -458,7 +523,7 @@ class RegexpTagger(SequentialBackoffTagger, yaml.YAMLObject):
         assigned the tag None.
     """
 
-    yaml_tag = '!nltk.RegexpTagger'
+    json_tag = 'nltk.tag.sequential.RegexpTagger'
 
     def __init__(self, regexps, backoff=None):
         """
@@ -468,9 +533,22 @@ class RegexpTagger(SequentialBackoffTagger, yaml.YAMLObject):
         tags = [tag for regex, tag in regexps]
         self._map = dict(zip(labels, tags))
         regexps_labels = [(regex, label) for ((regex,tag),label) in zip(regexps,labels)]
-        self._regexs = re.compile('|'.join(['(?P<%s>%s)' % (label, regex) for regex,label in regexps_labels]))
+        self._regexs = re.compile('|'.join('(?P<%s>%s)' % (label, regex) for regex,label in regexps_labels))
         self._size=len(regexps)
 
+    def encode_json_obj(self):
+        return self._map, self._regexs.pattern, self._size, self.backoff
+
+    @classmethod
+    def decode_json_obj(cls, obj):
+        _map, _regexs, _size, backoff = obj
+        self = cls(())
+        self._map = _map
+        self._regexs = re.compile(_regexs)
+        self._size = _size
+        SequentialBackoffTagger.__init__(self, backoff)
+        return self
+
     def choose_tag(self, tokens, index, history):
         m = self._regexs.match(tokens[index])
         if m:
@@ -480,6 +558,8 @@ class RegexpTagger(SequentialBackoffTagger, yaml.YAMLObject):
     def __repr__(self):
         return '<Regexp Tagger: size=%d>' % self._size
 
+
+@python_2_unicode_compatible
 class ClassifierBasedTagger(SequentialBackoffTagger, FeaturesetTaggerI):
     """
     A sequential tagger that uses a classifier to choose the tag for
@@ -558,13 +638,10 @@ class ClassifierBasedTagger(SequentialBackoffTagger, FeaturesetTaggerI):
         # higher than that cutoff first; otherwise, return None.
         if self._cutoff_prob is None:
             return self._classifier.classify(featureset)
-        else:
-            pdist = self._classifier.prob_classify(featureset)
-            tag = pdist.max()
-            if pdist.prob(tag) >= self._cutoff_prob:
-                return tag
-            else:
-                return None
+
+        pdist = self._classifier.prob_classify(featureset)
+        tag = pdist.max()
+        return tag if pdist.prob(tag) >= self._cutoff_prob else None
 
     def _train(self, tagged_corpus, classifier_builder, verbose):
         """
@@ -574,7 +651,7 @@ class ClassifierBasedTagger(SequentialBackoffTagger, FeaturesetTaggerI):
 
         classifier_corpus = []
         if verbose:
-            print 'Constructing training corpus for classifier.'
+            print('Constructing training corpus for classifier.')
 
         for sentence in tagged_corpus:
             history = []
@@ -586,7 +663,7 @@ class ClassifierBasedTagger(SequentialBackoffTagger, FeaturesetTaggerI):
                 history.append(tags[index])
 
         if verbose:
-            print 'Training classifier (%d instances)' % len(classifier_corpus)
+            print('Training classifier (%d instances)' % len(classifier_corpus))
         self._classifier = classifier_builder(classifier_corpus)
 
     def __repr__(self):
@@ -664,7 +741,3 @@ class ClassifierBasedPOSTagger(ClassifierBasedTagger):
         return features
 
 
-
-if __name__ == "__main__":
-    import doctest
-    doctest.testmod(optionflags=doctest.NORMALIZE_WHITESPACE)
